package example;
import models.Multinomial_Logistic_Regression;

/**
 * @author Marios Michailidis 2013
 * @Goal The purpose of this example class is to show you
 * how to use the Multinomial Logistic regerssion classification algorithm from
 * the KazAClasses package. In this example (similarly with the Naive Bayes example) we will try to predict
 * if someone lives in a small,medium or big house while we know:
 *<pre></pre> 1) The Age
 * <pre></pre>2) The Income
 * <pre></pre>3) A flag that shows whether they are married or not
 */


public class Multinomial_Logistic_example {

	/**
	 * @param args This is the main method that runs the example
	 */
	
	public static void main(String[] args) {
		
		/*
		 * In this example we will
		 * Initialize the names of the predictors we are going to use,
		 * Namely : Age, Income, if they are married.
		 */
		String [] Name_of_predictors = {"Age","Income","Marital_Status"};
		/*
		 * Initialize a double Array for the predictors again
		 * displaying Age, Income and a binary target that when it is 1=Are married
		 * and when it is 0= they are not.
		 */
		double predictors_array [][]= {{23,15000,0},
				{24,16000,0},{25,17000,0},{26,18000,0},{27,19000,1},{28,20000,0},
				{29,21000,0},{30,22000,1},{31,23000.34,0},{32,24000,0},{33,25000,1},
				{34,26000,1},{35,27000,1},{36,28000,0},{37,29000.78,0},{38,30000,1},
				{39,31000,1},{40,32000,0},{41,33000,1},{42,34000,0},{43,35000,1},
				{44,36000,0},{45,37000,1},{46,38000.17,0},{47,39000,1},{48,40000,0},
				{49,41000,1},{50,42000,0},{51,43000,1},{52,44000,0},{53,45000,1},
				{54,46000,1},{55,47000,1},{56,48000,0},{57,49000,1},{58,50000,1},
				{59,51000,0},{60,52000,1},{61,53000,1},{62,54340,0},{63,30034.78,1},
				{64,56000,1},{65,57000,0},{66,58000,0},{67,59000,1},{68,60000,0},
				{69,61000,1},{70,62000,1},{71,63000,1},{72,64000,0},{73,65000,0},
				{74,66000,1},{75,67000,1},{76,68000,0},{77,69000,1},{78,70000,0},
				{79,71000,1},{80,72000,1},{81,73000,0},{82,74000,1},{83,75000,0},
				{84,76000,1},{85,77000,0},{86,78000,0},{87,79000,0},{88,80000,1},
				{89,81000,1},{90,82000,1},{91,83000,0},{92,84000,1},{93,48000,0},
				{94,49000,1},{95,50000,1},{96,51000,0},{97,52000,1},{98,53000,0},
				{99,54000,1},{100,55000,1},{101,55000,1},{23,27000,0},{24,28000,0},
				{25,29000,1},{26,30000,0},{27,31000,0},{28,32000,1},{29,33000,1},
				{30,34000,0},{31,35000,0},{32,36000,0},{33,37000,1},{34,38000,1},
				{35,39000,0},{36,40000,1},{37,38000,0},{38,39000,1},{39,40000,1},
				{40,38000,0},{41,39000,0},{42,40000,1},{43,41000,1},{44,42000,1},
				{45,43000,0},
		};
		
		/*
		 * we initialize the target array which is a String 
		 * and holds the values of Small, Medium and Big depending on the
		 * size of the house they are living 
		 */
	String [] House_Size ={"Small","Small","Medium","Small","Small","Big","Medium",
			"Small","Medium","Medium","Small","Big","Small","Small","Medium","Big",
			"Small","Small","Medium","Small","Big","Small","Medium","Medium","Big",
			"Medium","Small","Medium","Medium","Small","Medium","Medium","Big",
			"Medium","Medium","Small","Medium","Medium","Big","Big","Medium","Small",
			"Big","Big","Medium","Big","Medium","Big","Small","Big","Medium","Big",
			"Big","Big","Medium","Medium","Big","Small","Big","Big","Big","Medium",
			"Big","Big","Medium","Medium","Big","Big","Small","Big","Big","Medium",
			"Big","Big","Small","Medium","Medium","Small","Medium","Medium","Big",
			"Small","Small","Medium","Small","Small","Big","Small","Small","Medium",
			"Medium","Small","Small","Medium","Medium","Small","Medium","Small",
			"Medium","Small","Big","Medium"};
	
         /*
          * We initialize the main class for Multinomial Logistic regression
          */
	
	Multinomial_Logistic_Regression multi= new Multinomial_Logistic_Regression();
	
	/*
	 * As explained in the class, the multinomial Logit model
	 * uses all the distinct values of the target variable but one to create separate models
	 * and then calculate the probability for all distinct values taking advantage of the fact that
	 * the sum of the probabilities for all categories should be one. When calculating the probabilities
	 * the category that was not used in the model is used as 'reference' to the others. The interpretation
	 * of the results is always based on the category that was left aside. We may want to set the category
	 * that will be left aside manually as it makes sense to compare it with the others. In that case lets put
	 * "Medium" .
	 */
	multi.setKvalue("Medium");
	    /*
	     * We initiate the main method of that class
	     * and we run it to create the classification logic.
	     */
	   multi.regression(predictors_array, House_Size, 0.001, 20);
	
		/*
		 * Now that we have created the logic we can get:
		 * 1) The coefficients of 2 categories that was used(small and Big)
		 * 2) The Wald statistics for these categories
		 * 3) The wald P_values
		 * 4) The names of the distinct values in the order they were used in the model
		 * (the first-0 category always refers to the category left aside, in this case "Medium".   
		 */

	   double Betas [][]=multi.getbetas();
	   double Walds  [][]=multi.getwalds();
	   double Wald_p_Values[][]= multi.getwaldpvalues();
	   String new_names[]=multi.get_distinct_values();
	   
	   /*
	    * lets print the results
	    */
	   for (int i=0; i<new_names.length-1; i++){
	   System.out.println("For " + new_names[i+1] + " : ");
	   System.out.println("Constant is : " + Betas[i][0] + " Wald : " + Walds[i][0] + " and Pvalue : " +  Wald_p_Values[i][0]);
	   for (int k=1; k < Betas[0].length; k++) {
		   System.out.println(Name_of_predictors[k-1] + " is : " + Betas[i][k] + " Wald : " + Walds[i][k] + " and Pvalue : " +  Wald_p_Values[i][k]);	     
	   }}
	   
	   /*
	    * It will print:
	    * For Big : 
          Constant is : -3.1041188813150944 Wald : 16.941829465874235 and Pvalue : 3.854284715631984E-5
          Age is : 0.003846946250510712 Wald : 0.04536170166086303 and Pvalue : 0.8313403262068235
          Income is : 4.511204830114201E-5 Wald : 3.559555417815684 and Pvalue : 0.05920359346658188
          Marital_Status is : -0.3098721740467406 Wald : 0.42774763844600994 and Pvalue : 0.5130962223425808
          For Small : 
          Constant is : 1.0722847002690163 Wald : 2.682078182857946 and Pvalue : 0.10148321422279594
          Age is : -0.018531511446007176 Wald : 0.7457220671060752 and Pvalue : 0.3878340387897016
          Income is : -2.6825408961389538E-5 Wald : 0.8719406809106416 and Pvalue : 0.35041860587331264
          Marital_Status is : 0.5211537256173336 Wald : 1.2171750016072702 and Pvalue : 0.2699157723889738
	    * 
	    */
	   
	   /*
	    * We can provide a new set (or the same one in this case-Training Set)
	    * and return the prediction as a String array.We can also provide a set
	    *  to derive the probabilities of each observation
        * to belong in each different category.
	    */
		   double [][] probability=multi.getprobabilites(predictors_array);
		   String classify []=multi.getclassification(predictors_array);

        /*
         * Print the results :
         */
		   for (int i=0; i < House_Size.length; i++) {
			   System.out.println("Real size: " + House_Size[i] + " Prediction: " + classify[i] + " " + new_names[0] + "_prob: " +(probability[i][0]*100) + "%"  + " " + new_names[1] + "_prob: " +(probability[i][1]*100) + "%"  + " " + new_names[2] + "_prob: " +(probability[i][2]*100) + "%");
		   }
         /*
          * It will print:
          * * Real size: Small Prediction: Small Medium_prob: 42.152074787446026% Big_prob: 4.064641365350279% Small_prob: 53.78328384720369%
           Real size: Small Prediction: Small Medium_prob: 43.091907408301964% Big_prob: 4.363767622646675% Small_prob: 52.544324969051374%
           Real size: Medium Prediction: Small Medium_prob: 44.02121565587188% Big_prob: 4.681559532097919% Small_prob: 51.29722481203021%
           Real size: Small Prediction: Small Medium_prob: 44.93785779650285% Big_prob: 5.018841791002693% Small_prob: 50.04330041249445%
           Real size: Small Prediction: Small Medium_prob: 34.74435315281298% Big_prob: 2.989249853266238% Small_prob: 62.26639699392078%
           Real size: Big Prediction: Small Medium_prob: 46.72436591375803% Big_prob: 5.755191760938061% Small_prob: 47.520442325303904%
           Real size: Medium Prediction: Medium Medium_prob: 47.589762431084885% Big_prob: 6.155913948495536% Small_prob: 46.25432362041957%
           Real size: Small Prediction: Small Medium_prob: 37.54057940713611% Big_prob: 3.7408200803535503% Small_prob: 58.718600512510335%
           Real size: Medium Prediction: Medium Medium_prob: 49.2536636552769% Big_prob: 7.026668214356116% Small_prob: 43.71966813036698%
           Real size: Medium Prediction: Medium Medium_prob: 50.04733613298606% Big_prob: 7.498041387901163% Small_prob: 42.45462247911277%
           Real size: Small Prediction: Small Medium_prob: 40.31345517190819% Big_prob: 4.652685905338303% Small_prob: 55.03385892275351%
           Real size: Big Prediction: Small Medium_prob: 41.22322885053965% Big_prob: 4.9964131125870415% Small_prob: 53.78035803687331%
           Real size: Small Prediction: Small Medium_prob: 42.12216303645554% Big_prob: 5.361540859867732% Small_prob: 52.51629610367673%
           Real size: Small Prediction: Medium Medium_prob: 52.91652309766579% Big_prob: 9.642921974878485% Small_prob: 37.440554927455736%
           Real size: Medium Prediction: Medium Medium_prob: 53.54665388100551% Big_prob: 10.247728047814169% Small_prob: 36.20561807118033%
           Real size: Big Prediction: Small Medium_prob: 44.731256794022094% Big_prob: 6.594439972505861% Small_prob: 48.674303233472045%
           Real size: Small Prediction: Small Medium_prob: 45.56394065513526% Big_prob: 7.054247899661767% Small_prob: 47.38181144520298%
           Real size: Small Prediction: Medium Medium_prob: 55.193913728095325% Big_prob: 12.233733817684753% Small_prob: 32.572352454219924%
           Real size: Medium Prediction: Medium Medium_prob: 47.15949264347494% Big_prob: 8.052371563962025% Small_prob: 44.78813579256302%
           Real size: Small Prediction: Medium Medium_prob: 56.07211159846012% Big_prob: 13.70692353794401% Small_prob: 30.220964863595878%
           Real size: Big Prediction: Medium Medium_prob: 48.6459699407098% Big_prob: 9.160660575767213% Small_prob: 42.19336948352299%
           Real size: Small Prediction: Medium Medium_prob: 56.759312817897126% Big_prob: 15.302255532603501% Small_prob: 27.93843164949938%
           Real size: Medium Prediction: Medium Medium_prob: 50.00442616394639% Big_prob: 10.385170113035782% Small_prob: 39.610403723017825%
           Real size: Medium Prediction: Medium Medium_prob: 57.245127287081324% Big_prob: 17.021013630367815% Small_prob: 25.733859082550865%
           Real size: Big Prediction: Medium Medium_prob: 51.21657209099227% Big_prob: 11.731158896999098% Small_prob: 37.05226901200863%
           Real size: Medium Prediction: Medium Medium_prob: 57.52164011113307% Big_prob: 18.862535337445514% Small_prob: 23.615824551421426%
           Real size: Small Prediction: Medium Medium_prob: 52.26522016601164% Big_prob: 13.202872700363857% Small_prob: 34.53190713362449%
           Real size: Medium Prediction: Medium Medium_prob: 57.583573832764536% Big_prob: 20.825366988095176% Small_prob: 21.5910591791403%
           Real size: Medium Prediction: Medium Medium_prob: 53.134733206879424% Big_prob: 14.80332924015975% Small_prob: 32.06193755296082%
           Real size: Small Prediction: Medium Medium_prob: 57.428541910315886% Big_prob: 22.90588523092641% Small_prob: 19.665572858757706%
           Real size: Medium Prediction: Medium Medium_prob: 53.81146060565486% Big_prob: 16.53411358116224% Small_prob: 29.654425813182904%
           Real size: Medium Prediction: Medium Medium_prob: 54.07390787411914% Big_prob: 17.448436384324616% Small_prob: 28.47765574155624%
           Real size: Big Prediction: Medium Medium_prob: 54.284144195741504% Big_prob: 18.395194078391334% Small_prob: 27.320661725867158%
           Real size: Medium Prediction: Medium Medium_prob: 56.47289064158245% Big_prob: 27.397427949202758% Small_prob: 16.12968140921479%
           Real size: Medium Prediction: Medium Medium_prob: 54.5442757208538% Big_prob: 20.384769279837574% Small_prob: 25.070954999308626%
           Real size: Small Prediction: Medium Medium_prob: 54.59280096691271% Big_prob: 21.426666959326415% Small_prob: 23.980532073760873%
           Real size: Medium Prediction: Medium Medium_prob: 55.21261657661023% Big_prob: 31.0238697199613% Small_prob: 13.763513703428467%
           Real size: Medium Prediction: Medium Medium_prob: 54.52486521220339% Big_prob: 23.601471726421135% Small_prob: 21.873663061375474%
           Real size: Big Prediction: Medium Medium_prob: 54.408268689211724% Big_prob: 24.732727489083217% Small_prob: 20.85900382170506%
           Real size: Big Prediction: Medium Medium_prob: 53.292145612363896% Big_prob: 35.21842733096053% Small_prob: 11.489427056675572%
           Real size: Medium Prediction: Medium Medium_prob: 54.153618703118745% Big_prob: 8.803257320899641% Small_prob: 37.043123975981615%
           Real size: Small Prediction: Medium Medium_prob: 53.73160667900565% Big_prob: 28.289469039705654% Small_prob: 17.9789242812887%
           Real size: Big Prediction: Medium Medium_prob: 51.44958715321323% Big_prob: 38.780665716129995% Small_prob: 9.769747130656773%
           Real size: Big Prediction: Medium Medium_prob: 50.68291574709731% Big_prob: 40.119690912260054% Small_prob: 9.19739334064264%
           Real size: Medium Prediction: Medium Medium_prob: 52.58089711769725% Big_prob: 32.06349239372007% Small_prob: 15.355610488582688%
           Real size: Big Prediction: Medium Medium_prob: 49.04975546053622% Big_prob: 42.821125286340965% Small_prob: 8.129119253122814%
           Real size: Medium Prediction: Medium Medium_prob: 51.566687300458995% Big_prob: 34.67985779251494% Small_prob: 13.753454907026077%
           Real size: Big Prediction: Medium Medium_prob: 50.9903623683983% Big_prob: 36.012958014643296% Small_prob: 12.996679616958398%
           Real size: Small Prediction: Medium Medium_prob: 50.37044721824119% Big_prob: 37.36019413862835% Small_prob: 12.269358643130456%
           Real size: Big Prediction: Big Medium_prob: 45.4523868115246% Big_prob: 48.26456629477537% Small_prob: 6.283046893700038%
           Real size: Medium Prediction: Big Medium_prob: 44.49870323431027% Big_prob: 49.6228484795668% Small_prob: 5.878448286122939%
           Real size: Big Prediction: Medium Medium_prob: 48.2707125810992% Big_prob: 41.46721715603069% Small_prob: 10.2620702628701%
           Real size: Big Prediction: Medium Medium_prob: 47.49855022861027% Big_prob: 42.85131524227548% Small_prob: 9.65013452911424%
           Real size: Big Prediction: Big Medium_prob: 41.5475581587482% Big_prob: 53.662104803044656% Small_prob: 4.79033703820715%
           Real size: Medium Prediction: Medium Medium_prob: 45.860705902348265% Big_prob: 45.62992761392067% Small_prob: 8.509366483731064%
           Real size: Medium Prediction: Big Medium_prob: 39.52960231078985% Big_prob: 56.30797202374437% Small_prob: 4.162425665465793%
           Real size: Big Prediction: Big Medium_prob: 44.11544740063037% Big_prob: 48.40887332700505% Small_prob: 7.47567927236458%
           Real size: Small Prediction: Big Medium_prob: 43.20912441353865% Big_prob: 49.79346789125847% Small_prob: 6.997407695202876%
           Real size: Big Prediction: Big Medium_prob: 36.47347955294298% Big_prob: 60.174501524137966% Small_prob: 3.3520189229190422%
           Real size: Big Prediction: Big Medium_prob: 41.342293244787676% Big_prob: 52.54322356994054% Small_prob: 6.114483185271777%
           Real size: Big Prediction: Big Medium_prob: 34.44156464911934% Big_prob: 62.667651517905654% Small_prob: 2.8907838329749955%
           Real size: Medium Prediction: Big Medium_prob: 39.42054557342133% Big_prob: 55.25480153388238% Small_prob: 5.324652892696278%
           Real size: Big Prediction: Big Medium_prob: 32.43205018674369% Big_prob: 65.08189541550928% Small_prob: 2.486054397747018%
           Real size: Big Prediction: Big Medium_prob: 31.439995049551545% Big_prob: 66.25686444890847% Small_prob: 2.3031405015399886%
           Real size: Medium Prediction: Big Medium_prob: 30.458482634113203% Big_prob: 67.40921903257632% Small_prob: 2.1322983333104824%
           Real size: Medium Prediction: Big Medium_prob: 35.491733910463516% Big_prob: 60.50971317818615% Small_prob: 3.9985529113503357%
           Real size: Big Prediction: Big Medium_prob: 34.505262036786945% Big_prob: 61.77970475705076% Small_prob: 3.7150332061623%
           Real size: Big Prediction: Big Medium_prob: 33.521343641005984% Big_prob: 63.029598116884024% Small_prob: 3.4490582421099805%
           Real size: Small Prediction: Big Medium_prob: 26.66448246771805% Big_prob: 71.77854904996242% Small_prob: 1.5569684823195273%
           Real size: Big Prediction: Big Medium_prob: 31.56884346106814% Big_prob: 65.46467765687551% Small_prob: 2.9664788820563466%
           Real size: Big Prediction: Medium Medium_prob: 58.71182361917068% Big_prob: 32.840633322920134% Small_prob: 8.447543057909188%
           Real size: Medium Prediction: Medium Medium_prob: 60.15239509739253% Big_prob: 25.919446557636743% Small_prob: 13.92815834497072%
           Real size: Big Prediction: Medium Medium_prob: 59.7443757724907% Big_prob: 27.035378160392526% Small_prob: 13.220246067116781%
           Real size: Big Prediction: Medium Medium_prob: 56.388030056481576% Big_prob: 36.530930950538384% Small_prob: 7.081038992980041%
           Real size: Small Prediction: Medium Medium_prob: 58.78343143302831% Big_prob: 29.336993001559296% Small_prob: 11.879575565412392%
           Real size: Medium Prediction: Medium Medium_prob: 54.66925732912786% Big_prob: 39.06089893511898% Small_prob: 6.269843735753157%
           Real size: Medium Prediction: Medium Medium_prob: 57.637733656634694% Big_prob: 31.724355022208083% Small_prob: 10.637911321157222%
           Real size: Small Prediction: Medium Medium_prob: 56.99927451769655% Big_prob: 32.94715012279982% Small_prob: 10.053575359503633%
           Real size: Medium Prediction: Medium Medium_prob: 57.03212630928992% Big_prob: 33.09320257509052% Small_prob: 9.87467111561957%
           Real size: Medium Prediction: Medium Medium_prob: 47.836610703733584% Big_prob: 7.926229480425037% Small_prob: 44.237159815841366%
           Real size: Big Prediction: Medium Medium_prob: 48.596623715090324% Big_prob: 8.456194407125105% Small_prob: 42.947181877784566%
           Real size: Small Prediction: Small Medium_prob: 39.11966510647625% Big_prob: 5.243856946625894% Small_prob: 55.63647794689787%
           Real size: Small Prediction: Medium Medium_prob: 50.02458210904055% Big_prob: 9.600140147114406% Small_prob: 40.375277743845054%
           Real size: Medium Prediction: Medium Medium_prob: 50.687924381046614% Big_prob: 10.215537740478723% Small_prob: 39.09653787847466%
           Real size: Small Prediction: Small Medium_prob: 41.726924367904786% Big_prob: 6.478283439984868% Small_prob: 51.79479219211035%
           Real size: Small Prediction: Small Medium_prob: 42.566205912622316% Big_prob: 6.940186378571934% Small_prob: 50.49360770880574%
           Real size: Big Prediction: Medium Medium_prob: 52.44895695237427% Big_prob: 12.242819435182616% Small_prob: 35.308223612443115%
           Real size: Small Prediction: Medium Medium_prob: 52.952741302424464% Big_prob: 12.980626611000549% Small_prob: 34.066632086574984%
           Real size: Small Prediction: Medium Medium_prob: 53.411711851355% Big_prob: 13.750114928579668% Small_prob: 32.83817322006532%
           Real size: Medium Prediction: Medium Medium_prob: 45.709472462011824% Big_prob: 9.064897250095068% Small_prob: 45.2256302878931%
           Real size: Medium Prediction: Medium Medium_prob: 46.429415606494665% Big_prob: 9.669689197043606% Small_prob: 43.90089519646172%
           Real size: Small Prediction: Medium Medium_prob: 54.50298023273763% Big_prob: 16.250919707209995% Small_prob: 29.246100060052378%
           Real size: Small Prediction: Medium Medium_prob: 47.77297575611057% Big_prob: 10.973036765794918% Small_prob: 41.25398747809451%
           Real size: Medium Prediction: Medium Medium_prob: 54.99503527384252% Big_prob: 15.795401901826528% Small_prob: 29.209562824330952%
           Real size: Medium Prediction: Medium Medium_prob: 48.169503162720716% Big_prob: 10.65776668155526% Small_prob: 41.17273015572401%
           Real size: Small Prediction: Medium Medium_prob: 48.79949104401097% Big_prob: 11.338926895919828% Small_prob: 39.861582060069196%
           Real size: Medium Prediction: Medium Medium_prob: 55.773764833811114% Big_prob: 16.205009109027372% Small_prob: 28.021226057161524%
           Real size: Small Prediction: Medium Medium_prob: 56.01431307784688% Big_prob: 17.09153048360033% Small_prob: 26.894156438552802%
           Real size: Medium Prediction: Medium Medium_prob: 49.807592769294864% Big_prob: 11.707504621953127% Small_prob: 38.48490260875201%
           Real size: Small Prediction: Medium Medium_prob: 50.3713036150067% Big_prob: 12.434106909419084% Small_prob: 37.19458947557421%
           Real size: Big Prediction: Medium Medium_prob: 50.89318006413763% Big_prob: 13.193305421937444% Small_prob: 35.91351451392492%
           Real size: Medium Prediction: Medium Medium_prob: 56.4460980752133% Big_prob: 20.949147288667806% Small_prob: 22.604754636118894%

         */
          
	}

}
