package example;
import Cluster.Naive_Bayes_classifier;

/**
 * @author Marios Michailidis 2013
 * @Goal The purpose of this example class is to show you
 * how to use the Naive Bayes classification algorithm from
 * the KazAClasses package. In this example we will try to predict
 * if someone lives in a small,medium or big house while we know:
 *<pre></pre> 1) The Age
 * <pre></pre>2) The Income
 * <pre></pre>3) A flag that shows whether they are married or not
 */
public class Naive_Bayes_Example {

	/**
	 * @param args This is the main method that runs the example
	 */
	public static void main(String[] args) {
	
		/*
		 * Initialize a double Array for the predictors again
		 * displaying Age, Income and a binary target that when it is 1=Are married
		 * and when it is 0= they are not.
		 */
		double predictors_array [][]= {{23,15000,0},
				{24,16000,0},{25,17000,0},{26,18000,0},{27,19000,1},{28,20000,0},
				{29,21000,0},{30,22000,1},{31,23000.34,0},{32,24000,0},{33,25000,1},
				{34,26000,1},{35,27000,1},{36,28000,0},{37,29000.78,0},{38,30000,1},
				{39,31000,1},{40,32000,0},{41,33000,1},{42,34000,0},{43,35000,1},
				{44,36000,0},{45,37000,1},{46,38000.17,0},{47,39000,1},{48,40000,0},
				{49,41000,1},{50,42000,0},{51,43000,1},{52,44000,0},{53,45000,1},
				{54,46000,1},{55,47000,1},{56,48000,0},{57,49000,1},{58,50000,1},
				{59,51000,0},{60,52000,1},{61,53000,1},{62,54340,0},{63,30034.78,1},
				{64,56000,1},{65,57000,0},{66,58000,0},{67,59000,1},{68,60000,0},
				{69,61000,1},{70,62000,1},{71,63000,1},{72,64000,0},{73,65000,0},
				{74,66000,1},{75,67000,1},{76,68000,0},{77,69000,1},{78,70000,0},
				{79,71000,1},{80,72000,1},{81,73000,0},{82,74000,1},{83,75000,0},
				{84,76000,1},{85,77000,0},{86,78000,0},{87,79000,0},{88,80000,1},
				{89,81000,1},{90,82000,1},{91,83000,0},{92,84000,1},{93,48000,0},
				{94,49000,1},{95,50000,1},{96,51000,0},{97,52000,1},{98,53000,0},
				{99,54000,1},{100,55000,1},{101,55000,1},{23,27000,0},{24,28000,0},
				{25,29000,1},{26,30000,0},{27,31000,0},{28,32000,1},{29,33000,1},
				{30,34000,0},{31,35000,0},{32,36000,0},{33,37000,1},{34,38000,1},
				{35,39000,0},{36,40000,1},{37,38000,0},{38,39000,1},{39,40000,1},
				{40,38000,0},{41,39000,0},{42,40000,1},{43,41000,1},{44,42000,1},
				{45,43000,0},
		};
		
		/*
		 * we initialize the target array which is a String 
		 * and holds the values of Small, Medium and Big depending on the
		 * size of the house they are living 
		 */
	String [] House_Size ={"Small","Small","Medium","Small","Small","Big","Medium",
			"Small","Medium","Medium","Small","Big","Small","Small","Medium","Big",
			"Small","Small","Medium","Small","Big","Small","Medium","Medium","Big",
			"Medium","Small","Medium","Medium","Small","Medium","Medium","Big",
			"Medium","Medium","Small","Medium","Medium","Big","Big","Medium","Small",
			"Big","Big","Medium","Big","Medium","Big","Small","Big","Medium","Big",
			"Big","Big","Medium","Medium","Big","Small","Big","Big","Big","Medium",
			"Big","Big","Medium","Medium","Big","Big","Small","Big","Big","Medium",
			"Big","Big","Small","Medium","Medium","Small","Medium","Medium","Big",
			"Small","Small","Medium","Small","Small","Big","Small","Small","Medium",
			"Medium","Small","Small","Medium","Medium","Small","Medium","Small",
			"Medium","Small","Big","Medium"};
	
         /*
          * We initialize the main class for Naive Bayes
          */
	
	Naive_Bayes_classifier naive= new Naive_Bayes_classifier();
	
	    /*
	     * We initiate the main method of that class
	     * and we run it to create the classification logic.
	     */
	naive.Create_Logic(predictors_array, House_Size);
	
	   /*
	    * We can provide a new set (or the same one in this case-Training Set)
	    * and return the prediction as a String array.
	    */
	   
   String House_Size_Prediction []=naive.Classify(predictors_array);
   
   /*
    * Lets compare reality with prediction
    */
   
   for (int i=0; i < House_Size.length; i++) {
	   System.out.println("The real house size: " + House_Size[i] + " and Prediction " + House_Size_Prediction[i] );
   }
   
   /*
    * We can also provide a set to derive the probabilities of each observation
    * to belong in each different cluster. First we need to derive an String that will
    * hold the order of the clusters as it was used from the algorithm
    */
   String clusters []=naive.get_target();
   
   /*
    * Now we get the probabilities of a new set
    */
   double Probabilities_for_each_cluster [] [] = naive.getprobabilities(predictors_array);
   
   /*
    * Lets compare reality with prediction and the probabilities
    */
	
   for (int i=0; i < House_Size.length; i++) {
	   System.out.println("Real size: " + House_Size[i] + " Prediction: " + House_Size_Prediction[i] + " " + clusters[0] + "_prob: " +(Probabilities_for_each_cluster[i][0]*100) + "%"  + " " + clusters[1] + "_prob: " +(Probabilities_for_each_cluster[i][1]*100) + "%"  + " " + clusters[2] + "_prob: " +(Probabilities_for_each_cluster[i][2]*100) + "%");
   }
   
   /*
    * It will print :
    * Real size: Small Prediction: Small Medium_prob: 21.583924975976622% Big_prob: 3.155651055326318% Small_prob: 75.26042396869707%
                Real size: Small Prediction: Small Medium_prob: 22.27773044387255% Big_prob: 3.3769598499949547% Small_prob: 74.3453097061325%
                Real size: Medium Prediction: Small Medium_prob: 22.98761305912182% Big_prob: 3.6155561919646058% Small_prob: 73.39683074891359%
                Real size: Small Prediction: Small Medium_prob: 23.712947122472283% Big_prob: 3.872780999722062% Small_prob: 72.41427187780566%
                Real size: Small Prediction: Small Medium_prob: 21.168773122442648% Big_prob: 3.6378754654137% Small_prob: 75.19335141214366%
                Real size: Big Prediction: Small Medium_prob: 25.20674474887451% Big_prob: 4.448935999600158% Small_prob: 70.34431925152535%
                Real size: Medium Prediction: Small Medium_prob: 25.973196280761513% Big_prob: 4.771013132510302% Small_prob: 69.25579058672818%
                Real size: Small Prediction: Small Medium_prob: 23.298413208123176% Big_prob: 4.513525936866744% Small_prob: 72.18806085501008%
                Real size: Medium Prediction: Small Medium_prob: 27.539002320938327% Big_prob: 5.491823757577721% Small_prob: 66.96917392148394%
                Real size: Medium Prediction: Small Medium_prob: 28.334891229875865% Big_prob: 5.894181787329345% Small_prob: 65.77092698279478%
                Real size: Small Prediction: Small Medium_prob: 25.546561710118493% Big_prob: 5.617311000928927% Small_prob: 68.83612728895258%
                Real size: Big Prediction: Small Medium_prob: 26.31606327142859% Big_prob: 6.045220861543943% Small_prob: 67.63871586702747%
                Real size: Small Prediction: Small Medium_prob: 27.092685294981568% Big_prob: 6.5068144876756495% Small_prob: 66.40050021734277%
                Real size: Small Prediction: Small Medium_prob: 31.56039575095481% Big_prob: 7.83222468880036% Small_prob: 60.60737956024482%
                Real size: Medium Prediction: Small Medium_prob: 32.36501494603766% Big_prob: 8.410264092273811% Small_prob: 59.22472096168854%
                Real size: Big Prediction: Small Medium_prob: 29.442704249411% Big_prob: 8.117994223310005% Small_prob: 62.439301527279%
                Real size: Small Prediction: Small Medium_prob: 30.22389639151179% Big_prob: 8.739088758046465% Small_prob: 61.03701485044174%
                Real size: Small Prediction: Small Medium_prob: 34.72310348305166% Big_prob: 10.405910656745194% Small_prob: 54.870985860203135%
                Real size: Medium Prediction: Small Medium_prob: 31.7641477418572% Big_prob: 10.123527594955409% Small_prob: 58.11232466318739%
                Real size: Small Prediction: Small Medium_prob: 36.21303649471487% Big_prob: 11.980232131896255% Small_prob: 51.80673137338888%
                Real size: Big Prediction: Small Medium_prob: 33.250058695809514% Big_prob: 11.716167718313976% Small_prob: 55.03377358587651%
                Real size: Small Prediction: Small Medium_prob: 37.59783928462559% Big_prob: 13.772774771259645% Small_prob: 48.62938594411477%
                Real size: Medium Prediction: Small Medium_prob: 34.64797945262148% Big_prob: 13.539078673290877% Small_prob: 51.81294187408765%
                Real size: Medium Prediction: Small Medium_prob: 38.84027411206098% Big_prob: 15.802311962209023% Small_prob: 45.357413925729986%
                Real size: Big Prediction: Small Medium_prob: 35.920352361727964% Big_prob: 15.613107744742633% Small_prob: 48.4665398935294%
                Real size: Medium Prediction: Small Medium_prob: 39.900715119979736% Big_prob: 18.08482777965502% Small_prob: 42.01445710036524%
                Real size: Small Prediction: Small Medium_prob: 37.026692013716435% Big_prob: 17.956462664449866% Small_prob: 45.0168453218337%
                Real size: Medium Prediction: Medium Medium_prob: 40.73925448459853% Big_prob: 20.633060299311516% Small_prob: 38.62768521608995%
                Real size: Medium Prediction: Small Medium_prob: 37.92509335728869% Big_prob: 20.58302356677605% Small_prob: 41.49188307593526%
                Real size: Small Prediction: Medium Medium_prob: 41.31705185166382% Big_prob: 23.453966596507595% Small_prob: 35.22898155182859%
                Real size: Medium Prediction: Medium Medium_prob: 38.574286288441236% Big_prob: 23.500472083764947% Small_prob: 37.92524162779382%
                Real size: Medium Prediction: Medium Medium_prob: 38.79332440373797% Big_prob: 25.06851442478467% Small_prob: 36.13816117147734%
                Real size: Big Prediction: Medium Medium_prob: 38.93617912044805% Big_prob: 26.708391132833214% Small_prob: 34.35542974671873%
                Real size: Medium Prediction: Medium Medium_prob: 41.55606365677437% Big_prob: 29.905619173301268% Small_prob: 28.538317169924355%
                Real size: Medium Prediction: Medium Medium_prob: 38.97873493242948% Big_prob: 30.19655372128696% Small_prob: 30.82471134628355%
                Real size: Small Prediction: Medium Medium_prob: 38.872634775275074% Big_prob: 32.03952481405871% Small_prob: 29.08784041066624%
                Real size: Medium Prediction: Medium Medium_prob: 40.841567046890724% Big_prob: 35.39632188938707% Small_prob: 23.762111063722216%
                Real size: Medium Prediction: Medium Medium_prob: 38.39655163107255% Big_prob: 35.90449785620696% Small_prob: 25.698950512720486%
                Real size: Big Prediction: Medium Medium_prob: 38.02536157115025% Big_prob: 37.91683563702316% Small_prob: 24.057802791826592%
                Real size: Big Prediction: Big Medium_prob: 39.182005494479675% Big_prob: 41.77002267823359% Small_prob: 19.04797182728673%
                Real size: Medium Prediction: Small Medium_prob: 36.75849671668667% Big_prob: 19.20496179751439% Small_prob: 44.036541485798935%
                Real size: Small Prediction: Big Medium_prob: 36.39075987486951% Big_prob: 44.201342144036374% Small_prob: 19.40789798109411%
                Real size: Big Prediction: Big Medium_prob: 37.08004933060029% Big_prob: 47.5748676025432% Small_prob: 15.345083066856521%
                Real size: Big Prediction: Big Medium_prob: 36.18063667889436% Big_prob: 49.68954356842442% Small_prob: 14.129819752681227%
                Real size: Medium Prediction: Big Medium_prob: 34.037796571989546% Big_prob: 50.70611336359998% Small_prob: 15.25609006441046%
                Real size: Big Prediction: Big Medium_prob: 34.19240328452457% Big_prob: 53.92760283358628% Small_prob: 11.879993881889158%
                Real size: Medium Prediction: Big Medium_prob: 32.135120460701% Big_prob: 55.059374120018404% Small_prob: 12.805505419280596%
                Real size: Big Prediction: Big Medium_prob: 31.102804987286664% Big_prob: 57.21658691829805% Small_prob: 11.680608094415279%
                Real size: Small Prediction: Big Medium_prob: 30.02615212035146% Big_prob: 59.35031207089888% Small_prob: 10.623535808749661%
                Real size: Big Prediction: Big Medium_prob: 29.647525107083567% Big_prob: 62.23202179222388% Small_prob: 8.12045310069255%
                Real size: Medium Prediction: Big Medium_prob: 28.435623236354886% Big_prob: 64.23162653823061% Small_prob: 7.332750225414503%
                Real size: Big Prediction: Big Medium_prob: 26.606402306595868% Big_prob: 65.53775949043961% Small_prob: 7.8558382029645255%
                Real size: Big Prediction: Big Medium_prob: 25.428722532414138% Big_prob: 67.50725524503008% Small_prob: 7.064022222555775%
                Real size: Big Prediction: Big Medium_prob: 24.73680179298066% Big_prob: 69.9505494778076% Small_prob: 5.312648729211733%
                Real size: Medium Prediction: Big Medium_prob: 23.060833624208627% Big_prob: 71.27406075880045% Small_prob: 5.665105616990929%
                Real size: Medium Prediction: Big Medium_prob: 22.286914050146837% Big_prob: 73.48316940834354% Small_prob: 4.2299165415096205%
                Real size: Big Prediction: Big Medium_prob: 20.72061176310618% Big_prob: 74.78408799395194% Small_prob: 4.495300242941873%
                Real size: Small Prediction: Big Medium_prob: 19.575700763216247% Big_prob: 76.43527551544051% Small_prob: 3.9890237213432425%
                Real size: Big Prediction: Big Medium_prob: 18.75401920084864% Big_prob: 78.29643102270771% Small_prob: 2.94954977644366%
                Real size: Big Prediction: Big Medium_prob: 17.36103962239914% Big_prob: 79.52097128401454% Small_prob: 3.1179890935863206%
                Real size: Big Prediction: Big Medium_prob: 16.546710131307428% Big_prob: 81.16120123206152% Small_prob: 2.2920886366310538%
                Real size: Medium Prediction: Big Medium_prob: 15.272641273596054% Big_prob: 82.31312004558713% Small_prob: 2.414238680816822%
                Real size: Big Prediction: Big Medium_prob: 14.488391914967808% Big_prob: 83.7463185881161% Small_prob: 1.7652894969160953%
                Real size: Big Prediction: Big Medium_prob: 13.520517401459664% Big_prob: 84.93526436895894% Small_prob: 1.5442182295814084%
                Real size: Medium Prediction: Big Medium_prob: 12.59535589965138% Big_prob: 86.05661326817533% Small_prob: 1.348030832173288%
                Real size: Medium Prediction: Big Medium_prob: 11.557679735709051% Big_prob: 87.03262912974351% Small_prob: 1.4096911345474477%
                Real size: Big Prediction: Big Medium_prob: 10.733304826473386% Big_prob: 88.0407540656855% Small_prob: 1.2259411078411169%
                Real size: Big Prediction: Big Medium_prob: 9.95163371201376% Big_prob: 88.98430225008484% Small_prob: 1.0640640379014004%
                Real size: Small Prediction: Big Medium_prob: 9.331965423971193% Big_prob: 89.90049170240833% Small_prob: 0.7675428736204688%
                Real size: Big Prediction: Big Medium_prob: 8.5153328964065% Big_prob: 90.68756728317305% Small_prob: 0.7970998204204456%
                Real size: Big Prediction: Big Medium_prob: 35.396110949953766% Big_prob: 55.76723228952633% Small_prob: 8.836656760519904%
                Real size: Medium Prediction: Big Medium_prob: 33.66841101313998% Big_prob: 56.720115063291296% Small_prob: 9.611473923568733%
                Real size: Big Prediction: Big Medium_prob: 32.802010454058845% Big_prob: 58.39922555155809% Small_prob: 8.798763994383066%
                Real size: Big Prediction: Big Medium_prob: 32.60916191566218% Big_prob: 60.63726538591211% Small_prob: 6.753572698425712%
                Real size: Small Prediction: Big Medium_prob: 30.964881508073326% Big_prob: 61.709209520042194% Small_prob: 7.325908971884484%
                Real size: Medium Prediction: Big Medium_prob: 30.60725653264137% Big_prob: 63.80549418564788% Small_prob: 5.58724928171075%
                Real size: Medium Prediction: Big Medium_prob: 29.019587168735615% Big_prob: 64.93296902081724% Small_prob: 6.047443810447156%
                Real size: Small Prediction: Big Medium_prob: 28.018064488786415% Big_prob: 66.50491192961543% Small_prob: 5.477023581598152%
                Real size: Medium Prediction: Big Medium_prob: 27.760087445690562% Big_prob: 66.98564628533025% Small_prob: 5.254266268979195%
                Real size: Medium Prediction: Small Medium_prob: 27.107669913985383% Big_prob: 4.5156097939321596% Small_prob: 68.37672029208247%
                Real size: Big Prediction: Small Medium_prob: 27.89719208844015% Big_prob: 4.8728926327010065% Small_prob: 67.22991527885885%
                Real size: Small Prediction: Small Medium_prob: 25.089491216463955% Big_prob: 4.6567068689740125% Small_prob: 70.25380191456203%
                Real size: Small Prediction: Small Medium_prob: 29.4981958011454% Big_prob: 5.678265808994737% Small_prob: 64.82353838985986%
                Real size: Medium Prediction: Small Medium_prob: 30.305484247298093% Big_prob: 6.131027664040197% Small_prob: 63.56348808866172%
                Real size: Small Prediction: Small Medium_prob: 27.396909835105294% Big_prob: 5.902880293226966% Small_prob: 66.70020987166774%
                Real size: Small Prediction: Small Medium_prob: 28.177750025198645% Big_prob: 6.390285449608117% Small_prob: 65.43196452519325%
                Real size: Big Prediction: Small Medium_prob: 32.72384359818794% Big_prob: 7.720243666161644% Small_prob: 59.555912735650416%
                Real size: Small Prediction: Small Medium_prob: 33.51865329399626% Big_prob: 8.336131919477813% Small_prob: 58.14521478652593%
                Real size: Small Prediction: Small Medium_prob: 34.30201083509347% Big_prob: 8.999923105900269% Small_prob: 56.69806605900628%
                Real size: Medium Prediction: Small Medium_prob: 31.290163384953185% Big_prob: 8.77661032505697% Small_prob: 59.93322628998985%
                Real size: Medium Prediction: Small Medium_prob: 32.04908909470407% Big_prob: 9.498076105295878% Small_prob: 58.45283480000004%
                Real size: Small Prediction: Small Medium_prob: 36.54274325327046% Big_prob: 11.308975758646561% Small_prob: 52.14828098808296%
                Real size: Small Prediction: Small Medium_prob: 33.51644649421737% Big_prob: 11.113977454101144% Small_prob: 55.36957605168148%
                Real size: Medium Prediction: Small Medium_prob: 36.62190316939919% Big_prob: 11.666993939797146% Small_prob: 51.71110289080367%
                Real size: Medium Prediction: Small Medium_prob: 33.611726346912% Big_prob: 11.458601964602622% Small_prob: 54.92967168848537%
                Real size: Small Prediction: Small Medium_prob: 34.30820473600231% Big_prob: 12.36980241956251% Small_prob: 53.32199284443517%
                Real size: Medium Prediction: Small Medium_prob: 37.39884101261649% Big_prob: 12.947700649837152% Small_prob: 49.65345833754636%
                Real size: Small Prediction: Small Medium_prob: 38.051317863315376% Big_prob: 13.91430435859267% Small_prob: 48.03437777809195%
                Real size: Medium Prediction: Small Medium_prob: 35.07433291320593% Big_prob: 13.727677583598402% Small_prob: 51.19798950319567%
                Real size: Small Prediction: Small Medium_prob: 35.70177040192139% Big_prob: 14.781362256272224% Small_prob: 49.5168673418064%
                Real size: Big Prediction: Small Medium_prob: 36.28841487926654% Big_prob: 15.905218769804124% Small_prob: 47.80636635092934%
                Real size: Medium Prediction: Small Medium_prob: 40.21205276013533% Big_prob: 18.4406419822334% Small_prob: 41.34730525763126%

    */
   
   /*
    * There is the option to take the means and variances used
    * in the logic.
    */
   double [][] means =naive.Get_means();
   double [][] variances =naive.Get_variances();
   

	}

}
